@@ -1,3 +1,21 @@
+/******************************************************************************\
+ *  Copyright (C) 2001, hexcurse is  writen by Jewfish and Armoth             *
+ *									      *
+ *  This program is free software; you can redistribute it and/or modify      *
+ *  it under the terms of the GNU General Public License as published by      *
+ *  the Free Software Foundation; either version 2 of the License, or	      *
+ *  (at your option) any later version.					      *
+ *									      *
+ *  This program is distributed in the hope that it will be useful,	      *
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of	      *
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	      *
+ *  GNU General Public License for more details.			      *
+ *									      *
+ *  You should have received a copy of the GNU General Public License	      *
+ *  along with this program; if not, write to the Free Software		      *
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
+ *									      *
+\******************************************************************************/
 #include "hex.h"
 
 /********************************************************\
@@ -5,38 +23,84 @@
  *		interprets it.				*
  * Returns:	retval (if quit program or no)		*
 \********************************************************/
-int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename, 
-	      char *fpOUTfilename, bool printHex)
+long maxlines;						/*extern val for lines*/
+long currentLine;					/*extern val for lines*/
+long LastLoc;                                           /*last cursor location*/
+bool editHex;						/* flag to edit h or a*/
+int SIZE_CH;                                            /* global winch flag  */
+
+int wacceptch(WINS *win, long len, char *fpINfilename, char *fpOUTfilename)
 {
-    int row, col, count, maxlines, val, tmpval, cl,	/* counters, etc.     */
-        ch[17],						/* holds search string*/
-	eol = (BASE * 3) - 1,				/* end of line pos    */
-	currentLine = 0,				/* current line       */
-        save = 0,					/* to save on exit    */
-	tmp;
+    int  row = 0, col = 0, count,  val, tmpval, 	/* counters, etc.     */
+         ch[17],					/* holds search string*/
+	 eol = (BASE * 3) - 1,				/* end of line pos    */
+         save = 0,					/* to save on exit    */
+	 lastRow = 0, lastCol = 0,			/* last row/col coords*/
+	 prevVal, curVal,				/* vals @ cursor locs */
+	 tmp;
 
-    long int gotoLoc = 0;				/* goto location      */
+    long cl,						/* current loc in file*/
+	 gotoLoc = 0,					/* goto location      */
+	 lastLine = 0,					/* line b4 LastLine   */
+	 currentLine = 0;				/* current line value */
 
-    char temp[80],
+    char *gotoLocStr,					/* convert to gotoLoc */
+         *temp,
+    	 *tmpstr,					/* tmp str 4 inputLine*/
 	 SearchStr[13];
 
     short int key;					/* key capture        */
-    WINDOW *windows;					/* window pointer     */
+    WINDOW *Winds;					/* window pointer     */
     hexStack *stack;					/* used for stack     */
     hexStack *tmpStack;					/* used for stack     */
-    bool editHex = TRUE;				/* val for editing    */
+    editHex = TRUE;					/* val for editing    */
 
-    maxlines = maxLines(len);				/* lines in file      */
-    windows = win->hex;					/* curr edit window   */
+    SIZE_CH	= FALSE;				/* set winch to false */
+    Winds       = win->hex;				/* curr edit window   */
+    maxlines    = maxLines(len);			/* lines in file      */
 
     /*createStack(stack);*/				/* init the stack     */
     stack = NULL;
-    bzero(temp, 80);					/* zero out string    */
+    temp = (char *)calloc(81, sizeof(char));
+
+    if (fpIN)						/* if file opened then*/
+    {							/* highlight 0,0 loc  */
+        prevVal = curVal = getLocVal(0);
+	wattron(win->ascii, A_UNDERLINE);
+	mvwprintw(win->ascii, 0, 0, "%c", curVal); 
+	wmove(win->hex, 0, 0);
+	wattrset(win->ascii, A_NORMAL);
+    	wnoutrefresh(win->ascii);
+	doupdate();
+    }
 
 							/* get keys til exit  */
-    while (!(save=quitProgram(isEmptyStack(stack),(key = wgetch(windows)))))
+    while (!(save=quitProgram(isEmptyStack(stack),(key = wgetch(Winds)))))
     {
-	getyx(windows, row, col);			/* curent cursor loc  */
+	lastRow = row;
+	lastCol = col;
+	getyx(Winds, row, col);				/* curent cursor loc  */
+
+	if (SIZE_CH)					/* if win size changed*/
+	{
+	    cl=row=col  = 0;
+	    eol         = (BASE * 3) - 1;
+	    editHex     = TRUE;
+	    Winds       = win->hex;
+	    /*cl = LastLoc;*/
+	    currentLine = 0;
+	    /*maxlines = maxLines(len);*/
+	    SIZE_CH = FALSE;
+	}
+
+	if ((COLS < MIN_COLS) || (LINES < MIN_LINES))	/* screen too small so*/
+	    continue;					/* ignore user input  */
+
+	/* if file isn't opened only allow user input from the following keys */
+	if (!fpIN && (key != CTRL_AND('o')) && (key != KEY_F(3)) &&
+	   (key != CTRL_AND('h')) && (key != KEY_F(1)))
+	    continue;
+	
 
 	switch (key) {					/* check keypress     */
 
@@ -44,11 +108,11 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 		if (currentLine > 0) 			/* move up...         */
 		{   
 		    currentLine--;
-		    wmove(windows, --row, col);
+		    wmove(Winds, --row, col);
 		}
 
 		if ((currentLine >= 0) && (row < 0))	/* scroll up...       */
-		    winscroll(win, windows, head, -1, currentLine, printHex);
+		    winscroll(win, Winds, -1, currentLine);
 		break;
 
 	case KEY_DOWN:					/* if DOWN...         */
@@ -56,12 +120,12 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 		{   
 		    if (currentLine < maxlines)		/* move down...       */
 		    {
-		        wmove(windows, ++row, col);
+		        wmove(Winds, ++row, col);
 		        currentLine++;
 		    }
 							/* scroll down...     */
 		    if((row > MAXY) && (currentLine <= maxlines))
-		        winscroll(win, windows, head, 1, currentLine, printHex);
+		        winscroll(win, Winds, 1, currentLine);
 		}
 		break;
 
@@ -72,17 +136,17 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 	  	    currentLine--;
 		    if (row == 0)			/* scroll up...       */
 		    {
-			winscroll(win, windows, head,-1, currentLine, printHex);
+			winscroll(win, Winds, -1, currentLine);
 			row++;
 		    }
-		    wmove(windows, --row, eol-1);
+		    wmove(Winds, --row, eol-1);
 		}
 
-		wmove(windows, row, --col);		/* move left...       */
+		wmove(Winds, row, --col);		/* move left...       */
 		if (editHex)
 		{
 		    if ((col + 1) % 3 == 0)
-		        wmove(windows, row, --col);
+		        wmove(Winds, row, --col);
 		}
 		break;
 
@@ -103,7 +167,7 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 			wattron(win->hex, A_BOLD);
 			wattron(win->ascii, A_BOLD);
 							/* output it          */
-			wprintw(windows, "%c", editHex ? toupper(key): key);
+			wprintw(Winds, "%c", editHex ? toupper(key): key);
 
 			tmpval = val;			/* val b4 key press   */
 
@@ -126,7 +190,8 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 			if (editHex)			/* update ascii win   */
 			{
 			    wmove(win->ascii, row, (col/3));	
-			    wprintw(win->ascii, "%c", isprint(val) ? val : 46);
+			    wprintw(win->ascii, "%c", (USE_EBCDIC) ?EBCDIC[val]:
+			           (isprint(val) ? val : 46));
 			    wmove(win->hex, row, col);
 			    wrefresh(win->ascii);
 			}
@@ -154,35 +219,34 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 							    case              */
 
 	case KEY_RIGHT:					/* if RIGHT...        */
-		    if (cursorLoc(currentLine, col, editHex,BASE) < len) 
+		    if (cursorLoc(currentLine, col, editHex, BASE) < len) 
 		    {   
-			wmove(windows, row, ++col);	/* move right         */
-		        if (cursorLoc(currentLine, col,editHex,BASE) == len)
-			    wmove(windows, row, --col);
+			wmove(Winds, row, ++col);	/* move right         */
+		        if (cursorLoc(currentLine, col, editHex,BASE) == len)
+			    wmove(Winds, row, --col);
 							/* move down          */
 		        if ((col == eol) && (currentLine < maxlines))
 		        {
 		            currentLine++;
 		            if (row < MAXY) {
                                 col = 0;
-	  	                wmove(windows, ++row, col);
+	  	                wmove(Winds, ++row, col);
                             }
 	                    else			/* scroll down        */
 	                    {
-			        winscroll(win, windows, head, 1, currentLine,
-				          printHex);
-	 		        wmove(windows, row, 0);
+			        winscroll(win, Winds, 1, currentLine);
+	 		        wmove(Winds, row, 0);
 	                    }
 		        }
 		        if (editHex)			/* adjust for hex win */
 		            if ((col + 1) % 3 == 0)
-		                wmove(windows, row, ++col);
+		                wmove(Winds, row, ++col);
 		    }
 
 							/* if end of file...  */
 							/* adjust properly    */
 		    if (cursorLoc(currentLine, col, editHex,BASE) == len)
-			wmove(windows, row, (col = col - 2));
+			wmove(Winds, row, (col = col - 2));
 		}
 		break;
 
@@ -190,7 +254,7 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 	case KEY_PGUP:					/* if KEY_PGUP...     */
 		if (currentLine == row) {		/* if first page      */
 		    currentLine = 0;			/* just move to top   */
-		    wmove(windows,0,0);
+		    wmove(Winds,0,0);
 		    break;
 		}
 		else
@@ -198,7 +262,7 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 
 	case CTRL_AND('d'):
 	case KEY_PGDN:					/* if KEY_PGDN...     */
-		getyx(windows, row, col);		/* current location   */
+		getyx(Winds, row, col);		/* current location   */
                                                         /* if EOF < page away */
 		if ((maxlines - currentLine) <= MAXY)
                 {                                       /* inc line til end   */
@@ -212,8 +276,8 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 							/* scroll down...     */
 	                if ((row > MAXY) && (currentLine <= maxlines))
                         {
-	                    winscroll(win, windows, head, 1, currentLine, printHex);
-			    row = MAXY;		/* last row/col               */
+	                    winscroll(win, Winds, 1, currentLine);
+			    row = MAXY;			/* last row/col       */
 		            col = (editHex) ? 3*((len % BASE)-1):(len % BASE)-1;
                             if (col < 0)
 		                col = (editHex) ? 3 * (BASE - 1):BASE - 1;
@@ -222,16 +286,16 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 							/* move to EOF        */
 		    if ((len - cursorLoc(currentLine, col,editHex,BASE) <= BASE)
                         && (currentLine < maxlines) && (row >= MAXY))
-		        winscroll(win, windows, head, 1, ++currentLine, printHex);
+		        winscroll(win, Winds, 1, ++currentLine);
 
 		    row += (maxlines - currentLine);	/* adj to last line   */
 		    currentLine = maxlines;
 							/* last column        */
 		    col = (editHex) ? 3 * ((len % BASE) - 1):(len % BASE) - 1;
                     if (col < 0)
-		        col = (editHex) ? 3 * (BASE - 1):BASE - 1;
+		        col = (editHex) ? 3 * (BASE - 1) : BASE - 1;
 
-		    wmove(windows,row,col);		/* move cursor        */
+		    wmove(Winds,row,col);		/* move cursor        */
 
  		    break;
 		}
@@ -244,9 +308,11 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 		    currentLine = 0;
 
 		val = currentLine - row;		/* get val            */
-		if (val < 0) {
+		if (val < 0) 
+		{
 		    row = val = 0;
-                    if (cursorLoc(currentLine, col, editHex,BASE) != cursorLoc(row, col, editHex,BASE))
+                    if (cursorLoc(currentLine, col, editHex,BASE) != 
+			cursorLoc(row, col, editHex,BASE))
                         row = currentLine;
                 }
 
@@ -256,7 +322,7 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 							/* output lines       */
 		for(count = 0; count <= MAXY && (count + val) <= maxlines;
 		    count++)
-		    outline(fpIN, win, head, (count + val), printHex);
+		    outline(fpIN, (count + val));
 
             	if (count < MAXY)
                     for (; count <= MAXY; count++) {
@@ -268,21 +334,68 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
                         wclrtoeol(win->ascii);
                     }
 		
-		wmove(windows, row, col);		/* restore cursor     */
+		wmove(Winds, row, col);			/* restore cursor     */
     		wnoutrefresh(win->ascii);
     		wnoutrefresh(win->address);
     		wnoutrefresh(win->hex);
-	
 		break;
+
+	case CTRL_AND('t'):
+	case KEY_HOME:					/* goto head of file  */
+		currentLine = cl = row = col = 0;	/*set vals to 1st line*/
+		wmove(win->hex, 0, 0);			/* position cursors   */
+		wmove(win->ascii, 0, 0);
+		wmove(win->address, 0, 0);
+
+		for (count = 0; count <= MAXY && count <= 
+		     maxLines(maxLoc(fpIN)); count++)
+		    outline(fpIN, count);
+
+		mvwprintw(windows->cur_address, 0, 0, "00000000"); 
+		wmove((editHex) ? win->hex : win->ascii, 0, 0);
+    		wnoutrefresh(win->ascii);
+    		wnoutrefresh(win->address);
+    		wnoutrefresh(win->hex);
+		break;
+		
+	case CTRL_AND('b'):
+	case KEY_END:					/* goto end of file   */
+		if (cursorLoc(currentLine, col, editHex, BASE)==maxLoc(fpIN)-1)
+		    break;				/* alread at oef      */
+		
+		/* if there's more than 1 screen, move to the last screenfull */
+		if ((maxlines - currentLine) >= MAXY)	/*if more than 1 scrn */
+		    currentLine = gotoLine(fpIN, 
+			  	  cursorLoc(currentLine, col, editHex, BASE),
+                                  maxLoc(fpIN)-(BASE*((len%BASE)?MAXY:MAXY+1)),
+				  maxlines, Winds);
+
+		getyx(Winds, row, col);			/* move to EOF        */
+		currentLine = gotoLine(fpIN, 
+				cursorLoc(currentLine, col, editHex, BASE),
+				maxLoc(fpIN)-1, maxlines, Winds);
+
+    		wnoutrefresh(win->ascii);
+    		wnoutrefresh(win->address);
+    		wnoutrefresh(win->hex);
+	   	break;
+
 	case CTRL_AND('o'):
 	case KEY_F(3):					/* if F3 or ^o...     */
+
+
 		if (openfile(win, fpINfilename))	/* open file          */
+		{
+		    if (fpIN)
+			prevVal = curVal = getLocVal(0);
+
 	 	    return TRUE;			/* TRUE if worked     */
+		}
 		break;
 
 	case CTRL_AND('s'):				/* if F2 or ^s...     */
 	case KEY_F(2):					/* save the file      */
-		savefile(win, head, fpINfilename, fpOUTfilename);
+		savefile(win, fpINfilename, fpOUTfilename);
 		break;
 
 	case CTRL_AND('f'):
@@ -314,16 +427,40 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 		    strcat(SearchStr, ")");
 		}
 
-		wmove(win->hex_outline, LINES-1, 19);   /* output prompt      */
+		wmove(win->hex_outline, LINES - 1, 19); /* output prompt      */
     		wclrtoeol(win->hex_outline);
     		mvwprintw(win->hex_outline, LINES - 1, 1, 
 		          "Enter %s value %s: ", 
 		          (editHex) ? "hex" : "ascii", SearchStr);
 
 		echo();					/* echo chars         */
-		wscanw(win->hex_outline, "%s", temp);
+		/*wscanw(win->hex_outline, "%s", temp);*/
+		/* the third parameter positions the cursor in the correct loc*/
+		tmpstr = inputLine(win->hex_outline, LINES - 1, 
+			 ((editHex) ? 21 : 23) + 
+			 ((strlen(temp) > 10) ? 10 : strlen(temp)));
 		noecho();
 
+		wmove(win->hex_outline, LINES - 1, 1);
+		wclrtoeol(win->hex_outline);
+    		mvwprintw(win->hex_outline, LINES - 1, 1, "Searching ...");
+		wrefresh(win->hex_outline);
+		doupdate();
+
+		if (tmpstr[0] == 27)			/* escape was hit     */
+		{					/* restore & return   */
+		    restoreBorder(win);
+		    wrefresh(win->hex_outline);
+		    break;
+		}
+
+		if (tmpstr[0] != '\0' )			/* enter was hit so   */
+		{					/* don't change temp  */
+		    bzero(temp, 81);
+		    strncpy(temp, tmpstr, (strlen(tmpstr) > 80) 
+			    ? 80 : strlen(tmpstr));
+		}
+
 		val = 0;
 							/* parse out input    */
 	        for (count = 0; temp[count] != 0 && count < 80; count++)
@@ -331,7 +468,8 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 			ch[count] = temp[count];
 		    else
 							/* if hex digit       */
-	 		if (isxdigit(key = temp[count])) {
+	 		if (isxdigit(key = temp[count])) 
+			{
 			    if (key >= 65 && key <= 70)	/* get correct val    */
 			    	key -= 7;
 			    else if (key >= 97 && key <= 102)
@@ -352,7 +490,7 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 		if (val != -1)				/* if val checks out  */
 							/* search for it      */
 		    val = hexSearch(fpIN, ch, cursorLoc(currentLine, col,
-			  editHex, BASE)+1, (editHex)?((count+1)/2):count,head);
+			  editHex, BASE)+1, (editHex) ? ((count+1)/2) : count);
 
 		if (val == -1) 				/* if nothing came up */
 		{
@@ -360,13 +498,13 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
                     restoreBorder(win);			/* restore border     */
 		    wrefresh(win->hex_outline);
 		}
-		else {
-                    getyx(windows, row, col);
+		else 
+		{
+                    getyx(Winds, row, col);
 							/* goto found loc     */
-                    currentLine = gotoLine(win, fpIN,
+                    currentLine = gotoLine(fpIN,
                                         cursorLoc(currentLine,col,editHex,BASE),
-                                           val, maxlines, head, printHex,
-                                           editHex, windows);
+                                           val, maxlines, Winds);
 
 		}
 		break;
@@ -376,14 +514,20 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 	case KEY_F(6):					/* if F6, ^a, ^d...   */
 		printHex = (!printHex);			/* reverse printHex   */
 
-		getyx(windows, row, col);		/* current location   */
+		getyx(Winds, row, col);			/* current location   */
 		row = currentLine - row;		/* compute top line   */
 
 		wmove(win->address, 0, 0);		/* move to origin     */
 							/* write out values   */
-		for(count = 0; count <= MAXY && count <= maxlines ; count++)
-		    wprintw(win->address, (printHex) ? "%08X ":"%08d ",
-			   (count + row) * BASE);
+		/* check to see if screen is filled     */
+		if ((maxlines - row) < MAXY)
+		    for(count = 0; count <= (maxlines - row); count++)
+			wprintw(win->address, (printHex) ? "%08X ":"%08d ",
+			       (count + row) * BASE);
+		else
+		    for(count = 0; count <= MAXY && count <= maxlines ; count++)
+			wprintw(win->address, (printHex) ? "%08X ":"%08d ",
+			       (count + row) * BASE);
 
 							/* update menu button */
 		slk_set(6, (printHex) ? "Hex Addr":"Dec Addr", 1);
@@ -399,7 +543,17 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 		"Enter %s location: ", (printHex) ? "HEX":"decimal");
 
 		echo();					/* echo chars         */
-		wscanw(win->hex_outline, (printHex) ? "%X":"%d",&gotoLoc);
+		gotoLocStr = inputLine(win->hex_outline, LINES - 1, 
+			              (printHex) ? 21 : 25);
+		if (gotoLocStr[0] == 27)		/* escape was hit     */
+		{					/* restore & return   */
+		    restoreBorder(win);
+		    wrefresh(win->hex_outline);
+		    break;
+		}
+
+		gotoLoc = strtol(gotoLocStr, NULL, 16); /* convert str 2 hex  */
+		/*wscanw(win->hex_outline, (printHex) ? "%X":"%d",&gotoLoc);  */
 		noecho();				/* disable echoing    */
 
 		if ((gotoLoc < 0) || (gotoLoc > len-1))
@@ -408,49 +562,70 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
                     restoreBorder(win);			/* restore border     */
 		    wrefresh(win->hex_outline);
 		}
-		else {
-                    getyx(windows, row, col);
+		else 
+		{
+                    getyx(Winds, row, col);
 							/* goto found loc     */
-                    currentLine = gotoLine(win, fpIN,
-                                           cursorLoc(currentLine, col, editHex,BASE),
-                                           gotoLoc, maxlines, head, printHex,
-                                           editHex, windows);
+                    currentLine = gotoLine(fpIN,
+                                           cursorLoc(currentLine, col, editHex,
+					   BASE), gotoLoc, maxlines, Winds);
 	 	}
 		break;
 
 	case KEY_TAB:
 	case KEY_F(7):					/* if F7, TAB, ^i...  */
+							/*switch the underline*/
+		getyx(Winds, row, col);			/* current location   */
+		wattrset((editHex) ? win->ascii : win->hex, A_NORMAL);
+		mvwprintw((editHex) ? win->ascii : win->hex, row,
+			  (editHex) ? col/3 : col*3, 
+			  (editHex) ? "%c": "%02X", 
+			  (editHex) ? ((isprint(curVal))?curVal : '.'):curVal);
+		wnoutrefresh((editHex) ? win->ascii : win->hex);
 		if (editHex)				/* already in hex win */
 		{
-		    windows = win->ascii;		/* change to ascii    */
+		    Winds = win->ascii;			/* change to ascii    */
 		    eol = BASE;
-	 	    wmove(windows, row, (col/3));
+	 	    wmove(Winds, row, (col/3));
 		    slk_set(7, "Asc Edit", 1);
 		}
 		else					/* already in ascii   */
 		{
-		    windows = win->hex;			/* change to hex win  */
+		    Winds = win->hex;			/* change to hex win  */
 		    eol = (BASE * 3) - 1;
-	 	    wmove(windows, row, (col*3));	
+	 	    wmove(Winds, row, (col*3));	
 		    slk_set(7, "Hex Edit", 1);
 		}
 
 		editHex = (!editHex);			/* change test val    */
 		slk_noutrefresh();			/* refresh menu       */
+
+		getyx(Winds, row, col);			/* current location   */
+		/* re-bold char if the value over the cursor is modified      */
+		if (inHexList(cursorLoc(currentLine, col, editHex, BASE))) 
+		{
+		    wattron((editHex) ? win->hex : win->ascii, A_BOLD);
+		    mvwprintw((editHex) ? win->hex : win->ascii , row, col, 
+			      (editHex) ? "%02X" : "%c", 
+			      (editHex) ?curVal:((isprint(curVal))?curVal:'.'));
+		    wattrset((editHex) ? win->hex : win->ascii, A_NORMAL);
+	 	    wmove(Winds, row, col);	
+		    wnoutrefresh((editHex) ? win->ascii : win->hex);
+		}
 		break;
 
 	case CTRL_AND('?'):
 	case CTRL_AND('h'):
 	case KEY_F(1):					/* if F1, ^?, ^h...   */
-		getyx(windows, row, col);		/* current location   */
+		getyx(Winds, row, col);			/* current location   */
 
 		printHelp(win);				/* display the help   */
 
-	 	wmove(windows, row, col);		/* restore cursor     */
+	 	wmove(Winds, row, col);			/* restore cursor     */
 		break;
 
 	case CTRL_AND('z'):				/* ^z undo last mod   */
-		getyx(windows, row, col);
+		getyx(Winds, row, col);
 
 							/* set previous loc   */
 		cl = (stack == NULL) ? cl : stack->currentLoc;
@@ -463,10 +638,9 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 		    head = deleteNode(head,cl);
 
 
-                    currentLine = gotoLine(win, fpIN, 
-			  	    	cursorLoc(currentLine, col, editHex,BASE),
-                                    	cl, maxlines, head, printHex,
-                                    	editHex, windows);
+                    currentLine = gotoLine(fpIN, 
+			  	    cursorLoc(currentLine, col, editHex,BASE),
+                                    cl, maxlines, Winds);
 		/*
                     if (stack != NULL)
 		    { 
@@ -478,7 +652,7 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 		    }
 		*/
 		
-		    getyx(windows, row, col);
+		    getyx(Winds, row, col);
 
    		    if ((searchList(head, cl)) != -1)
 		    {	
@@ -488,7 +662,8 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
                     if (editHex)
 		    {
                         wmove(win->ascii, row, (col/3));    
-                        wprintw(win->ascii, "%c", isprint(val) ? val : 46);
+			wprintw(win->ascii, "%c", (USE_EBCDIC) ? EBCDIC[val] :
+			       (isprint(val) ? val : 46));
                         wmove(win->hex, row, col);
                         wrefresh(win->ascii);
 
@@ -503,7 +678,8 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
                         wmove(win->ascii, row, col);
                         wrefresh(win->hex);
 
-                        wprintw(win->ascii, "%c", isprint(val) ? val : 46);
+			wprintw(win->ascii, "%c", (USE_EBCDIC) ? EBCDIC[val] :
+			       (isprint(val) ? val : 46));
                         wmove(win->ascii, row, col);    
                         wrefresh(win->ascii);
 		    }
@@ -521,22 +697,69 @@ int wacceptch(WINS *win, hexList *head, long int len, char *fpINfilename,
 #endif
 	}
 
-	getyx(windows, row, col);			/* get cur row/col    */
-    
+	getyx(Winds, row, col);				/* get cur row/col    */
+	if (fpIN)
+	{
+							/* store current loc  */
+	    /* remove underline from previous character */
+	    if ((lastRow == MAXY) && (key == KEY_DOWN) && maxlines!=currentLine)
+		lastRow--;
+	    else if ((lastRow == 0) && (key == KEY_UP) && currentLine != 0)
+		lastRow++;
+	    if ((lastRow == MAXY) && (lastCol == eol-1) && (key == KEY_RIGHT) &&
+		(maxlines != currentLine))
+		lastRow--;
+	    else if ((lastRow == 0) && (lastCol == 0) && (key == KEY_LEFT) &&
+		    (currentLine != 0))
+		lastRow++;
+	    
+
+	    if (key != KEY_TAB)				/* if tab, don't do it*/
+	    {   /* unhighlight the previous character if tab was not pressed  */
+	        curVal = getLocVal(cursorLoc(lastLine, lastCol, editHex, BASE));
+		wattrset((editHex) ? win->ascii : win->hex, A_NORMAL);
+		wattron((editHex) ? win->ascii : win->hex,
+		  (inHexList(cursorLoc(lastLine, lastCol, editHex, BASE))) ?
+		  A_BOLD : A_NORMAL);
+		mvwprintw((editHex) ? win->ascii : win->hex, lastRow, 
+			  (editHex) ? lastCol/3 : lastCol*3, 
+			  (editHex) ? "%c" : "%02X", 
+			  (editHex) ? (isprint(curVal)) ? curVal : '.': curVal);
+		wmove(Winds, row, col);
+		wattrset((editHex) ? win->ascii : win->hex, A_NORMAL);
+		wnoutrefresh((editHex) ? win->ascii : win->hex);
+	    }
+							/* highlight new char */
+	    wattron((editHex) ? win->ascii : win->hex, A_UNDERLINE);
+	    if (inHexList(cursorLoc(currentLine, col, editHex, BASE)))
+	        wattron((editHex) ? win->ascii : win->hex, A_BOLD);
+
+	    curVal = getLocVal(cursorLoc(currentLine, col, editHex, BASE));
+
+	    mvwprintw((editHex) ? win->ascii : win->hex, row, 
+		      (editHex) ? col/3 : col*3,  
+		      (editHex) ? "%c" : "%02X", 
+		      (editHex) ? (isprint(curVal)) ? curVal : '.' : curVal); 
+	    wattrset((editHex) ? win->ascii : win->hex, A_NORMAL);
+	    wnoutrefresh((editHex) ? win->ascii : win->hex);
+	}
+							/* save last location */
+	LastLoc = cursorLoc(currentLine, col, editHex, BASE);
+	lastLine = currentLine;
 							/* print cur location */
 	mvwprintw(win->cur_address, 0, 0, (printHex) ? "%08X":"%08d",
 	   	  cursorLoc(currentLine, col, editHex,BASE));
 	
 	wnoutrefresh(win->cur_address);			/* refresh outline    */
 	
-        scrollbar(win, currentLine, maxlines);		/* change scroll bar  */
+	scrollbar(win, cursorLoc(currentLine, col, editHex,BASE), maxlines);
 	
-	wmove(windows, row, col);			/* restore cursor     */
+	wmove(Winds, row, col);				/* restore cursor     */
 	doupdate();					/* update visual      */
     }
 
     /* if user chose to save on exit, value will == 2 */
-    if (save == 2) savefile(win, head, fpINfilename, fpOUTfilename);
+    if (save == 2) savefile(win, fpINfilename, fpOUTfilename);
 
     return 0;						/* return             */
 }
@@ -571,24 +794,34 @@ char *inputLine(WINDOW *win, int line, int col)
 
     wmove(win, line, col);
 
-    for (x = 0; (c = wgetch(win)) != 10; x++) {
+    for (x = 0; (c = wgetch(win)) != 10; x++) 
+    {
         wclrtoeol(win);					/* clear line         */
-        if (c == '\b' || c == 127) {			/* get backspace      */
+        if (c == '\b' || c == 127) 			/* get backspace      */
+	{
             mvwaddch(win, line, col + (x-1), 32);
             wmove(win, line, col + (x-1));
             ch[x] = '\0';
             x -= 2;					/* modify ptr         */
         }
-        else if (c > 32 && c < 127) {			/* if printable char  */
+        else if (c > 32 && c < 127) 			/* if printable char  */
+	{
             ch[x] = c;
             waddch(win, ch[x]);
         }
-        else {						/* if anything else   */
+	else if (c == 27)				/* if the escape key  */
+	{   						/* is pressed, return */
+	    ch[0] = 27;					/* setting ch to 0xff */
+	    return ch;					
+	}
+        else 						/* if anything else   */
+	{
             ch[x] = '\0';
             x--;
         }
 
-        if (x < 0) {
+        if (x < 0) 
+	{
             wmove(win, line, col);			/* move cursor        */
             x = -1;
         }
